import { Injectable } from '@nestjs/common';
import { ConfigurationRepositoryPort } from '../core/application/ports/configuration.repository.port';
import { InjectRedis } from '@songkeys/nestjs-redis';
import { Redis } from 'ioredis';
import {
  ConfigurationIdentifier,
  ConfigurationValue,
} from '../core/domain/configuration.types';
import { ConfigurationEntity } from '../core/domain/configuration.entity';
import { ConfigurationMapper } from '../configuration.mapper';
import { NotFoundException } from '@mobicoop/ddd-library';

export type ConfigurationReadModel = {
  key: string;
  value: string;
};
export type ConfigurationWriteModel = ConfigurationReadModel;

@Injectable()
export class ConfigurationRepository implements ConfigurationRepositoryPort {
  constructor(
    @InjectRedis() private readonly redis: Redis,
    private readonly mapper: ConfigurationMapper,
  ) {}

  get = async (
    identifier: ConfigurationIdentifier,
  ): Promise<ConfigurationEntity> => {
    const key: string = `${identifier.domain}:${identifier.key}`;
    const value: ConfigurationValue | null = await this.redis.get(key);
    if (!value)
      throw new NotFoundException(
        `Configuration item not found for key ${key}`,
      );
    return this.mapper.toDomain({
      key,
      value,
    });
  };

  set = async (
    identifier: ConfigurationIdentifier,
    value: ConfigurationValue,
  ): Promise<void> => {
    await this.redis.set(`${identifier.domain}:${identifier.key}`, value);
  };
}
