import { Test, TestingModule } from '@nestjs/testing';
import { ConfigurationEntity } from '@modules/configuration/core/domain/configuration.entity';
import { ConfigurationDomain } from '@modules/configuration/core/domain/configuration.types';
import { GetConfigurationQueryHandler } from '@modules/configuration/core/application/queries/get-configuration/get-configuration.query-handler';
import { CONFIGURATION_REPOSITORY } from '@modules/configuration/configuration.di-tokens';
import { GetConfigurationQuery } from '@modules/configuration/core/application/queries/get-configuration/get-configuration.query';

const now = new Date('2023-06-21 06:00:00');
const configuration: ConfigurationEntity = new ConfigurationEntity({
  id: 'c160cf8c-f057-4962-841f-3ad68346df44',
  props: {
    identifier: {
      domain: ConfigurationDomain.CARPOOL,
      key: 'seatsProposed',
    },
    value: '3',
  },
  createdAt: now,
  updatedAt: now,
});

const mockConfigurationRepository = {
  get: jest.fn().mockImplementation(() => configuration),
};

describe('Get Configuration Query Handler', () => {
  let getConfigurationQueryHandler: GetConfigurationQueryHandler;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: CONFIGURATION_REPOSITORY,
          useValue: mockConfigurationRepository,
        },
        GetConfigurationQueryHandler,
      ],
    }).compile();

    getConfigurationQueryHandler = module.get<GetConfigurationQueryHandler>(
      GetConfigurationQueryHandler,
    );
  });

  it('should be defined', () => {
    expect(getConfigurationQueryHandler).toBeDefined();
  });

  describe('execution', () => {
    it('should return a configuration item', async () => {
      const getConfigurationQuery = new GetConfigurationQuery(
        ConfigurationDomain.CARPOOL,
        'seatsProposed',
      );
      const configuration: ConfigurationEntity =
        await getConfigurationQueryHandler.execute(getConfigurationQuery);
      expect(configuration.getProps().value).toBe('3');
    });
  });
});
