import { RpcExceptionCode } from '@mobicoop/ddd-library';
import { ConfigurationDomain } from '@modules/configuration/core/domain/configuration.types';
import { SetConfigurationRequestDto } from '@modules/configuration/interface/grpc-controllers/dtos/set-configuration.request.dto';
import { SetConfigurationGrpcController } from '@modules/configuration/interface/grpc-controllers/set-configuration.grpc.controller';
import { CommandBus } from '@nestjs/cqrs';
import { RpcException } from '@nestjs/microservices';
import { Test, TestingModule } from '@nestjs/testing';

const setConfigurationRequest: SetConfigurationRequestDto = {
  domain: ConfigurationDomain.CARPOOL,
  key: 'seatsProposed',
  value: '3',
};

const mockCommandBus = {
  execute: jest
    .fn()
    .mockImplementationOnce(() => '200d61a8-d878-4378-a609-c19ea71633d2')
    .mockImplementationOnce(() => {
      throw new Error();
    }),
};

describe('Set Configuration Grpc Controller', () => {
  let setConfigurationGrpcController: SetConfigurationGrpcController;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: CommandBus,
          useValue: mockCommandBus,
        },
        SetConfigurationGrpcController,
      ],
    }).compile();

    setConfigurationGrpcController = module.get<SetConfigurationGrpcController>(
      SetConfigurationGrpcController,
    );
  });

  afterEach(async () => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(setConfigurationGrpcController).toBeDefined();
  });

  it('should set a configuration item', async () => {
    jest.spyOn(mockCommandBus, 'execute');
    await setConfigurationGrpcController.set(setConfigurationRequest);
    expect(mockCommandBus.execute).toHaveBeenCalledTimes(1);
  });

  it('should throw a generic RpcException', async () => {
    jest.spyOn(mockCommandBus, 'execute');
    expect.assertions(3);
    try {
      await setConfigurationGrpcController.set(setConfigurationRequest);
    } catch (e: any) {
      expect(e).toBeInstanceOf(RpcException);
      expect(e.error.code).toBe(RpcExceptionCode.UNKNOWN);
    }
    expect(mockCommandBus.execute).toHaveBeenCalledTimes(1);
  });
});
